@@ -160,11 +160,14 @@ void DPrintPreviewWidgetPrivate::populateScene()
     scene->setSceneRect(QRect(QPoint(0, 0), paperSize));
 }
 
+void DPrintPreviewWidgetPrivate::updatePreview()
+{
+    generatePreview();
+    graphicsView->updateGeometry();
+}
+
 void DPrintPreviewWidgetPrivate::generatePreview()
 {
-    if (refreshMode == RefreshDelay)
-        return;
-
     int totalPages = 0;
     if (isAsynPreview) {
         if (currentPageNumber == 0) {
@@ -1211,10 +1214,22 @@ QVector<int> DPrintPreviewWidgetPrivate::requestPages(int page)
 }
 
 /*!
- * \~chinese \brief 构造一个 DPrintPreviewWidget。
- *
- * \~chinese \param printer 打印机
- * \~chinese \param parent 父控件
+    \class Dtk::Widget::DPrintPreviewWidget
+    \inmodule dtkwidget
+
+    打印预览对话框界面中的视图预览窗口，用于预览来自第三方应用中的
+    数据。通常情况下应用无需调用该类中的接口，只需要在 DPrintPreviewDialog
+    中处理即可。除此之外，该类还管理了将预览数据输出到打印机的功能，例如，另存为
+    PDF、另存为图片等功能。
+
+    \sa Dtk::Widget::DPrintPreviewDialog
+*/
+
+/*!
+  \brief 构造一个 DPrintPreviewWidget。
+  
+  \a printer 打印机
+  \a parent 父控件
  */
 DPrintPreviewWidget::DPrintPreviewWidget(DPrinter *printer, QWidget *parent)
     : DFrame(*new DPrintPreviewWidgetPrivate(this))
@@ -1228,24 +1243,24 @@ DPrintPreviewWidget::~DPrintPreviewWidget()
 {
     Q_D(DPrintPreviewWidget);
 
+    d->updateTimer.stop();
     delete d->numberUpPrintData;
 }
 
 /*!
- * \~chinese \brief 设置打印预览widget是否可见。
- *
- * \~chinese \param visible 是否可见
+  \brief 设置打印预览widget是否可见.
+  \reimp
+  
+  \a visible 是否可见
  */
 void DPrintPreviewWidget::setVisible(bool visible)
 {
     QWidget::setVisible(visible);
-    if (visible) {
-        updatePreview();
-    }
+    // updatePreview() function move into DPrintPreviewDialog show event.
 }
 
 /*!
- * \~chinese \brief 设置打印预览页面范围为所有页。
+  \brief 设置打印预览页面范围为所有页.
  */
 void DPrintPreviewWidget::setPageRangeALL()
 {
@@ -1256,9 +1271,9 @@ void DPrintPreviewWidget::setPageRangeALL()
 }
 
 /*!
- * \~chinese \brief 设置是否需要重新生成页面内容。
- *
- * \~chinese \param generate 是否需要重新生成页面内容
+  \brief 设置是否需要重新生成页面内容.
+  
+  \a generate 是否需要重新生成页面内容
  */
 void DPrintPreviewWidget::setReGenerate(bool generate)
 {
@@ -1266,9 +1281,9 @@ void DPrintPreviewWidget::setReGenerate(bool generate)
 }
 
 /*!
- * \~chinese \brief 设置页面选择范围模式。
- *
- * \~chinese \param mode 页面选择范围模式，AllPage所有页，CurrentPage当前页，SelectPage选择页
+  \brief 设置页面选择范围模式。
+  
+  \a mode 页面选择范围模式，AllPage所有页，CurrentPage当前页，SelectPage选择页
  */
 void DPrintPreviewWidget::setPageRangeMode(PageRange mode)
 {
@@ -1277,7 +1292,9 @@ void DPrintPreviewWidget::setPageRangeMode(PageRange mode)
 }
 
 /*!
- * \~chinese \brief 获取页面选择范围模式。
+  \brief 获取页面选择范围模式.
+
+  \return 返回页码返回的模式.
  */
 DPrintPreviewWidget::PageRange DPrintPreviewWidget::pageRangeMode()
 {
@@ -1286,7 +1303,9 @@ DPrintPreviewWidget::PageRange DPrintPreviewWidget::pageRangeMode()
 }
 
 /*!
- * \~chinese \brief 预览是否改变，当预览改变时需要调用。
+  \brief 预览是否改变，当预览改变时需要调用.
+
+  \a generate 预览是否发生改变.
  */
 void DPrintPreviewWidget::reviewChange(bool generate)
 {
@@ -1295,9 +1314,9 @@ void DPrintPreviewWidget::reviewChange(bool generate)
 }
 
 /*!
- * \~chinese \brief 设置预览页面范围。
- *
- * \~chinese \param rangePages 页码Vector
+  \brief 设置预览页面范围。
+  
+  \a rangePages 页码Vector
  */
 void DPrintPreviewWidget::setPageRange(const QVector<int> &rangePages)
 {
@@ -1320,10 +1339,10 @@ void DPrintPreviewWidget::setPageRange(const QVector<int> &rangePages)
 }
 
 /*!
- * \~chinese \brief 设置预览页面范围。
- *
- * \~chinese \param from 起始页码
- * \~chinese \param to 终止页码
+  \brief 设置预览页面范围。
+  
+  \a from 起始页码
+  \a to 终止页码
  */
 void DPrintPreviewWidget::setPageRange(int from, int to)
 {
@@ -1338,7 +1357,9 @@ void DPrintPreviewWidget::setPageRange(int from, int to)
 }
 
 /*!
- * \~chinese \brief 获取预览总页数。
+  \brief 获取预览总页数.
+
+  \return 返回预览总页数.
  */
 int DPrintPreviewWidget::pagesCount()
 {
@@ -1347,7 +1368,9 @@ int DPrintPreviewWidget::pagesCount()
 }
 
 /*!
- * \~chinese \brief 获取当前页的页码。
+  \brief 获取当前页的页码.
+
+  \return 返回当前页的页码.
  */
 int DPrintPreviewWidget::currentPage()
 {
@@ -1356,7 +1379,9 @@ int DPrintPreviewWidget::currentPage()
 }
 
 /*!
- * \~chinese \brief 获取是否可翻页。
+  \brief 获取是否可翻页.
+
+  \return 可翻页返回true,不可翻页返回false.
  */
 bool DPrintPreviewWidget::turnPageAble()
 {
@@ -1365,11 +1390,11 @@ bool DPrintPreviewWidget::turnPageAble()
 }
 
 /*!
- * \~chinese \brief 设置色彩模式。
- *
- * \~chinese \param colorMode 色彩模式
+  \brief 设置色彩模式。
+  
+  \a colorMode 色彩模式
  */
-void DPrintPreviewWidget::setColorMode(const QPrinter::ColorMode &colorMode)
+void DPrintPreviewWidget::setColorMode(const DPrinter::ColorMode &colorMode)
 {
     Q_D(DPrintPreviewWidget);
 
@@ -1388,20 +1413,20 @@ void DPrintPreviewWidget::setColorMode(const QPrinter::ColorMode &colorMode)
 }
 
 /*!
- * \~chinese \brief 设置页面方向。
- *
- * \~chinese \param pageOrientation 页面方向
+  \brief 设置页面方向。
+  
+  \a pageOrientation 页面方向
  */
-void DPrintPreviewWidget::setOrientation(const QPrinter::Orientation &pageOrientation)
+void DPrintPreviewWidget::setOrientation(const DPrinter::Orientation &pageOrientation)
 {
     Q_D(DPrintPreviewWidget);
 
     d->previewPrinter->setOrientation(pageOrientation);
-    d->generatePreview();
+    updatePreview();
 }
 
 /*!
- * \~chinese \brief 获取色彩模式。
+  \brief 获取色彩模式。
  */
 DPrinter::ColorMode DPrintPreviewWidget::getColorMode()
 {
@@ -1410,9 +1435,9 @@ DPrinter::ColorMode DPrintPreviewWidget::getColorMode()
 }
 
 /*!
- * \~chinese \brief 设置页面缩放。
- *
- * \~chinese \param scale 缩放大小
+  \brief 设置页面缩放。
+  
+  \a scale 缩放大小
  */
 void DPrintPreviewWidget::setScale(qreal scale)
 {
@@ -1424,7 +1449,7 @@ void DPrintPreviewWidget::setScale(qreal scale)
 }
 
 /*!
- * \~chinese \brief 获取缩放大小。
+  \brief 获取缩放大小。
  */
 qreal DPrintPreviewWidget::getScale() const
 {
@@ -1433,7 +1458,7 @@ qreal DPrintPreviewWidget::getScale() const
 }
 
 /*!
- * \~chinese \brief 刷新预览页面。
+  \brief 刷新预览页面。
  */
 void DPrintPreviewWidget::updateView()
 {
@@ -1453,7 +1478,7 @@ void DPrintPreviewWidget::updateView()
 }
 
 /*!
- * \~chinese \brief 强制刷新水印效果
+  \brief 强制刷新水印效果
  */
 void DPrintPreviewWidget::updateWaterMark()
 {
@@ -1469,7 +1494,7 @@ void DPrintPreviewWidget::updateWaterMark()
 }
 
 /*!
- * \~chinese \brief 水印刷新开始，主要用于同时多次调用水印属性改变的接口时，只刷新一次
+  \brief 水印刷新开始，主要用于同时多次调用水印属性改变的接口时，只刷新一次
  */
 void DPrintPreviewWidget::refreshBegin()
 {
@@ -1479,7 +1504,7 @@ void DPrintPreviewWidget::refreshBegin()
 }
 
 /*!
- * \~chinese \brief 水印刷新结束，刷新水印效果
+  \brief 水印刷新结束，刷新水印效果
  */
 void DPrintPreviewWidget::refreshEnd()
 {
@@ -1490,9 +1515,9 @@ void DPrintPreviewWidget::refreshEnd()
 }
 
 /*!
- * \~chinese \brief 设置水印类型。
- *
- * \~chinese \param type 水印类型
+  \brief 设置水印类型。
+  
+  \a type 水印类型
  */
 void DPrintPreviewWidget::setWaterMarkType(int type)
 {
@@ -1510,9 +1535,9 @@ void DPrintPreviewWidget::setWaterMarkType(int type)
 }
 
 /*!
- * \~chinese \brief 设置水印图片。
- *
- * \~chinese \param image 水印图片
+  \brief 设置水印图片。
+  
+  \a image 水印图片
  */
 void DPrintPreviewWidget::setWaterMargImage(const QImage &image)
 {
@@ -1530,9 +1555,9 @@ void DPrintPreviewWidget::setWaterMargImage(const QImage &image)
 }
 
 /*!
- * \~chinese \brief 设置水印旋转角度。
- *
- * \~chinese \param rotate 水印旋转角度
+  \brief 设置水印旋转角度。
+  
+  \a rotate 水印旋转角度
  */
 void DPrintPreviewWidget::setWaterMarkRotate(qreal rotate)
 {
@@ -1548,9 +1573,9 @@ void DPrintPreviewWidget::setWaterMarkRotate(qreal rotate)
 }
 
 /*!
- * \~chinese \brief 设置水印缩放大小。
- *
- * \~chinese \param rotate 水印缩放大小
+  \brief 设置水印缩放大小。
+  
+  \a scale 水印缩放大小
  */
 void DPrintPreviewWidget::setWaterMarkScale(qreal scale)
 {
@@ -1568,9 +1593,9 @@ void DPrintPreviewWidget::setWaterMarkScale(qreal scale)
 }
 
 /*!
- * \~chinese \brief 设置水印透明度。
- *
- * \~chinese \param rotate 水印透明度
+  \brief 设置水印透明度。
+  
+  \a opacity 水印透明度
  */
 void DPrintPreviewWidget::setWaterMarkOpacity(qreal opacity)
 {
@@ -1586,7 +1611,7 @@ void DPrintPreviewWidget::setWaterMarkOpacity(qreal opacity)
 }
 
 /*!
- * \~chinese \brief 设置“绝密”文字水印。
+  \brief 设置“绝密”文字水印。
  */
 void DPrintPreviewWidget::setConfidentialWaterMark()
 {
@@ -1604,7 +1629,7 @@ void DPrintPreviewWidget::setConfidentialWaterMark()
 }
 
 /*!
- * \~chinese \brief 设置“草稿”文字水印。
+  \brief 设置“草稿”文字水印。
  */
 void DPrintPreviewWidget::setDraftWaterMark()
 {
@@ -1622,7 +1647,7 @@ void DPrintPreviewWidget::setDraftWaterMark()
 }
 
 /*!
- * \~chinese \brief 设置“样本”文字水印。
+  \brief 设置“样本”文字水印。
  */
 void DPrintPreviewWidget::setSampleWaterMark()
 {
@@ -1640,9 +1665,9 @@ void DPrintPreviewWidget::setSampleWaterMark()
 }
 
 /*!
- * \~chinese \brief 设置自定义文字水印。
- *
- * \~chinese \param text 自定义文字
+  \brief 设置自定义文字水印。
+  
+  \a text 自定义文字
  */
 void DPrintPreviewWidget::setCustomWaterMark(const QString &text)
 {
@@ -1660,9 +1685,9 @@ void DPrintPreviewWidget::setCustomWaterMark(const QString &text)
 }
 
 /*!
- * \~chinese \brief 设置文字水印的文字内容。
- *
- * \~chinese \param text 文字水印的文字内容
+  \brief 设置文字水印的文字内容。
+  
+  \a text 文字水印的文字内容
  */
 void DPrintPreviewWidget::setTextWaterMark(const QString &text)
 {
@@ -1680,9 +1705,9 @@ void DPrintPreviewWidget::setTextWaterMark(const QString &text)
 }
 
 /*!
- * \~chinese \brief 设置文字水印的字体。
- *
- * \~chinese \param font 文字水印的字体
+  \brief 设置文字水印的字体。
+  
+  \a font 文字水印的字体
  */
 void DPrintPreviewWidget::setWaterMarkFont(const QFont &font)
 {
@@ -1700,9 +1725,25 @@ void DPrintPreviewWidget::setWaterMarkFont(const QFont &font)
 }
 
 /*!
- * \~chinese \brief 设置文字水印的颜色。
- *
- * \~chinese \param color 文字水印的颜色
+  \brief 获取文字水印的颜色。
+
+  \return 文字水印的颜色
+ */
+QColor DPrintPreviewWidget::waterMarkColor() const
+{
+    Q_D(const DPrintPreviewWidget);
+
+    if (imposition() == DPrintPreviewWidget::One) {
+        return d->waterMark->getColor();
+    } else {
+        return d->numberUpPrintData->waterProperty->color;
+    }
+}
+
+/*!
+  \brief 设置文字水印的颜色。
+  
+  \a color 文字水印的颜色
  */
 void DPrintPreviewWidget::setWaterMarkColor(const QColor &color)
 {
@@ -1720,9 +1761,9 @@ void DPrintPreviewWidget::setWaterMarkColor(const QColor &color)
 }
 
 /*!
- * \~chinese \brief 设置水印的布局。
- *
- * \~chinese \param layout 水印的布局，Center居中，Tiled平铺
+  \brief 设置水印的布局。
+  
+  \a layout 水印的布局，Center居中，Tiled平铺
  */
 void DPrintPreviewWidget::setWaterMarkLayout(int layout)
 {
@@ -1740,9 +1781,9 @@ void DPrintPreviewWidget::setWaterMarkLayout(int layout)
 }
 
 /*!
- * \~chinese \brief 设置并打的模式。
- *
- * \~chinese \param im 并打的模式
+  \brief 设置并打的模式。
+  
+  \a im 并打的模式
  */
 void DPrintPreviewWidget::setImposition(Imposition im)
 {
@@ -1791,9 +1832,9 @@ DPrintPreviewWidget::Imposition DPrintPreviewWidget::imposition() const
 }
 
 /*!
- * \~chinese \brief 设置页面内并打的顺序。
- *
- * \~chinese \param order 页面内并打的顺序
+  \brief 设置页面内并打的顺序。
+  
+  \a order 页面内并打的顺序
  */
 void DPrintPreviewWidget::setOrder(Order order)
 {
@@ -1816,7 +1857,6 @@ void DPrintPreviewWidget::setOrder(Order order)
 
         d->generatePreview();
         Q_EMIT pagesCountChanged(d->pageRange.count());
-
     } else {
         d->order = order;
         totalPage = pagesCount();
@@ -1907,17 +1947,18 @@ QByteArray DPrintPreviewWidget::printerColorModel() const
 }
 
 /*!
- * \~chinese \brief 刷新预览。
+  \brief 刷新预览。
  */
 void DPrintPreviewWidget::updatePreview()
 {
     Q_D(DPrintPreviewWidget);
-    d->generatePreview();
-    d->graphicsView->updateGeometry();
+
+    if (!d->updateTimer.isActive())
+        d->updateTimer.start(0, this);
 }
 
 /*!
- * \~chinese \brief 前翻一页。
+  \brief 前翻一页。
  */
 void DPrintPreviewWidget::turnFront()
 {
@@ -1928,7 +1969,7 @@ void DPrintPreviewWidget::turnFront()
 }
 
 /*!
- * \~chinese \brief 后翻一页。
+  \brief 后翻一页。
  */
 void DPrintPreviewWidget::turnBack()
 {
@@ -1939,7 +1980,7 @@ void DPrintPreviewWidget::turnBack()
 }
 
 /*!
- * \~chinese \brief 第一页。
+  \brief 第一页。
  */
 void DPrintPreviewWidget::turnBegin()
 {
@@ -1950,7 +1991,7 @@ void DPrintPreviewWidget::turnBegin()
 }
 
 /*!
- * \~chinese \brief 最后一页。
+  \brief 最后一页。
  */
 void DPrintPreviewWidget::turnEnd()
 {
@@ -1961,9 +2002,9 @@ void DPrintPreviewWidget::turnEnd()
 }
 
 /*!
- * \~chinese \brief 设置当前页。
- *
- * \~chinese \param page 当前页
+  \brief 设置当前页。
+  
+  \a page 当前页
  */
 void DPrintPreviewWidget::setCurrentPage(int page)
 {
@@ -1975,8 +2016,9 @@ void DPrintPreviewWidget::setCurrentPage(int page)
         int lastPage = d->index2page(d->currentPageNumber - 1);
         if (lastPage > 0)
             d->pages.at(lastPage - 1)->setVisible(false);
-    } else
+    } else if (!d->pages.isEmpty()) {
         d->pages.first()->setVisible(false);
+    }
     d->setCurrentPageNumber(page);
     if (d->isAsynPreview) {
         d->previewPages = d->requestPages(page);
@@ -2024,6 +2066,19 @@ void DPrintPreviewWidget::themeTypeChanged(DGuiApplicationHelper::ColorType them
         d->graphicsView->setBackgroundBrush(QColor(255, 255, 255, 120));
 }
 
+void DPrintPreviewWidget::timerEvent(QTimerEvent *event)
+{
+    Q_D(DPrintPreviewWidget);
+    if (event->timerId() == d->updateTimer.timerId()) {
+        if (d->updateTimer.isActive()) {
+            d->updateTimer.stop();
+            d->updatePreview();
+        }
+    }
+
+    return DFrame::timerEvent(event);
+}
+
 DPrinter::DPrinter(QPrinter::PrinterMode mode)
     : QPrinter(mode)
 {
@@ -2214,8 +2269,25 @@ void WaterMark::updatePicture(QPainter *painter, bool isPreview)
 
         QFontMetrics fm(font);
         QSize textSize = fm.size(Qt::TextSingleLine, text);
-        int space = qMin(textSize.width(), textSize.height());
-        QSize spaceSize = QSize(WATER_TEXTSPACE, space) * numberUpScale * wScale;
+
+        // TODO: Remove it.
+        QVariant spacingProperty = pwidget->property("_d_print_waterMarkRowSpacing");
+        int rowSpace;
+        if (spacingProperty.isValid()) {
+            rowSpace = qRound(textSize.height() * spacingProperty.toDouble());
+        } else {
+            rowSpace = WATER_TEXTSPACE;
+        }
+
+        int columnSpace;
+        spacingProperty = pwidget->property("_d_print_waterMarkColumnSpacing");
+        if (spacingProperty.isValid()) {
+            columnSpace = qRound(textSize.width() * spacingProperty.toDouble());
+        } else {
+            columnSpace = qMin(textSize.width(), textSize.height());
+        }
+
+        QSize spaceSize = QSize(columnSpace, rowSpace) * numberUpScale * wScale;
         QImage textImage(textSize + spaceSize, QImage::Format_ARGB32);
         textImage.fill(Qt::transparent);
         QPainter tp;
