@@ -148,7 +148,7 @@ private:
     bool                autoHideOnFullscreen = false;
     bool                fullScreenButtonVisible = true;
     bool                splitScreenWidgetEnable = true;
-
+    QTimer              *maxButtonPressAndHoldTimer = nullptr;
     Q_DECLARE_PUBLIC(DTitlebar)
 };
 
@@ -177,7 +177,7 @@ void DSplitScreenButton::initStyleOption(DStyleOptionButton *option) const
         QColor iconForeColor = Qt::white;
         if (DGuiApplicationHelper::instance()->themeType() == DGuiApplicationHelper::LightType) {
             backgroundBrush = DStyle::adjustColor(backgroundBrush, 0, 0, 0, 0, 0, 0, 3);
-            iconForeColor = dpal.textTiele().color();
+            iconForeColor = dpal.brush(DPalette::TextTitle).color();
         }
 
         pal.setBrush(QPalette::Light, backgroundBrush);
@@ -313,11 +313,15 @@ bool DSplitScreenWidget::eventFilter(QObject *o, QEvent *e)
     case QEvent::WindowDeactivate:
     case QEvent::FocusIn:
     case QEvent::FocusOut:
-    case QEvent::MouseButtonRelease:
     case QEvent::MouseButtonDblClick:
     case QEvent::Wheel:
         hideImmediately();
         break;
+    case QEvent::MouseButtonRelease:
+        if (!isMaxButtonPressAndHold) {
+            hideImmediately();
+        }
+        break;
     default:
         break;
     }
@@ -375,6 +379,8 @@ void DTitlebarPrivate::init()
     minButton       = new DWindowMinButton;
     maxButton       = new DWindowMaxButton;
     closeButton     = new DWindowCloseButton;
+    maxButtonPressAndHoldTimer = new QTimer(q);
+    maxButtonPressAndHoldTimer->setSingleShot(true);
 
     if (DGuiApplicationHelper::isTabletEnvironment()) {
         optionButton = new DTabletWindowOptionButton;
@@ -495,6 +501,11 @@ void DTitlebarPrivate::init()
     q->connect(DGuiApplicationHelper::instance()->systemTheme(), &DPlatformTheme::iconThemeNameChanged, q, [ = ]() {
         iconLabel->update();
     });
+    q->connect(maxButtonPressAndHoldTimer, &QTimer::timeout, q, [this]() {
+        showSplitScreenWidget();
+        if (splitWidget && splitWidget->isVisible())
+            splitWidget->isMaxButtonPressAndHold = true;
+    });
 
     // 默认需要构造一个空的选项菜单
     q->setMenu(new QMenu(q));
@@ -506,8 +517,13 @@ void DTitlebarPrivate::init()
     // 另外，让标题栏接收焦点，还是为了避免一个focus控件隐藏时，会把焦点转移给标题栏上的按钮控件
     q->setFocusPolicy(Qt::StrongFocus);
 
+    auto noTitlebarEnabled = []{
+        QFunctionPointer enableNoTitlebar = qApp->platformFunction("_d_isEnableNoTitlebar");
+        bool enabled = qApp->platformName() == "dwayland" || qApp->property("_d_isDwayland").toBool();
+        return enabled && enableNoTitlebar != nullptr;
+    };
     // fix wayland 下显示了两个应用图标，系统标题栏 和 dtk标题栏 均显示应用图标
-    q->setEmbedMode(!(DApplication::isDXcbPlatform()|| (qApp->platformName() == "dwayland" || qApp->property("_d_isDwayland").toBool())));
+    q->setEmbedMode(!(DApplication::isDXcbPlatform()|| noTitlebarEnabled()));
 }
 
 QWidget *DTitlebarPrivate::targetWindow()
@@ -632,9 +648,7 @@ void DTitlebarPrivate::updateButtonsFunc()
         return;
     }
     if (!qgetenv("WAYLAND_DISPLAY").isEmpty()) {
-        if (disableFlags.testFlag(Qt::WindowCloseButtonHint)) {
-            closeButton->setEnabled(false);
-        }
+        closeButton->setEnabled(!disableFlags.testFlag(Qt::WindowCloseButtonHint));
         return;
     }
 
@@ -696,6 +710,11 @@ void DTitlebarPrivate::handleParentWindowIdChange()
 
 void DTitlebarPrivate::_q_toggleWindowState()
 {
+    if (splitWidget && splitWidget->isMaxButtonPressAndHold) {
+        splitWidget->isMaxButtonPressAndHold = false;
+        return;
+    }
+
     QWidget *parentWindow = targetWindow();
 
     if (!parentWindow || disableFlags.testFlag(Qt::WindowMaximizeButtonHint)) {
@@ -997,10 +1016,14 @@ void DTitlebarPrivate::hideSplitScreenWidget()
 
 void DTitlebarPrivate::changeWindowSplitedState(quint32 type)
 {
-    //### 目前接口尚不公开在 dtkgui 中，等待获取后续接口稳定再做移植
+    if (splitWidget && splitWidget->isVisible()) {
+        splitWidget->isMaxButtonPressAndHold = false;
+        splitWidget->hideImmediately();
+    }
     D_Q(DTitlebar);
     QFunctionPointer splitWindowOnScreen = Q_NULLPTR;
 
+    //### 目前接口尚不公开在 dtkgui 中，等待获取后续接口稳定再做移植
     splitWindowOnScreen = qApp->platformFunction(CHANGESPLITWINDOW_VAR);
 
     const QWindow *windowHandle = nullptr;
@@ -1287,6 +1310,14 @@ bool DTitlebar::eventFilter(QObject *obj, QEvent *event)
             d->hideSplitScreenWidget();
             break;
         }
+        case QEvent::MouseButtonPress: {
+            d->maxButtonPressAndHoldTimer->start(300);
+            break;
+        }
+        case QEvent::MouseButtonRelease: {
+            d->maxButtonPressAndHoldTimer->stop();
+            break;
+        }
         default:
             break;
 
