@@ -22,7 +22,7 @@
 #ifndef DPRINTPREVIEWWIDGET_P_H
 #define DPRINTPREVIEWWIDGET_P_H
 
-#include "dprintpreviewwidget.h"
+#include <dprintpreviewwidget.h>
 #include "private/dframe_p.h"
 
 #include <DIconButton>
@@ -32,6 +32,7 @@
 #include <QWheelEvent>
 #include <QPicture>
 #include <qmath.h>
+#include <QBasicTimer>
 
 DWIDGET_BEGIN_NAMESPACE
 
@@ -165,6 +166,10 @@ public:
     {
         color = c;
     }
+    inline QColor getColor() const
+    {
+        return color;
+    }
     inline void setBoundingRect(const QRectF &rect)
     {
         qreal rotate = rotation();
@@ -264,6 +269,7 @@ public:
 
     void init();
     void populateScene();
+    void updatePreview();
     void generatePreview();
     void fitView();
     void print(bool printAsPicture = false);
@@ -335,6 +341,7 @@ public:
 
     struct NumberUpData;
     NumberUpData *numberUpPrintData;
+    QBasicTimer updateTimer;
     Q_DECLARE_PUBLIC(DPrintPreviewWidget)
 };
 
@@ -553,8 +560,25 @@ struct DPrintPreviewWidgetPrivate::NumberUpData {
         if (waterList.isEmpty())
             return;
 
-        for (auto *item : qAsConst(waterList))
-            outFunction(item);
+        auto *firstWm = waterList.first();
+        outFunction(firstWm);
+
+        for (auto *item : qAsConst(waterList)) {
+            if (item == firstWm)
+                continue;
+
+            // TODO: remove it in dtkwidget 5.6.
+            item->type = firstWm->type;
+            item->layout = firstWm->layout;
+            item->mScaleFactor = firstWm->mScaleFactor;
+            item->color = firstWm->color;
+            item->text = firstWm->text;
+            item->sourceImage = firstWm->sourceImage;
+            item->graySourceImage = firstWm->graySourceImage;
+            item->font = firstWm->font;
+            item->setRotation(firstWm->rotation());
+            item->setOpacity(firstWm->opacity());
+        }
     }
 
     NumberUpData(DPrintPreviewWidgetPrivate *parent)
